@@ -19,8 +19,8 @@
  *  \file
  *  \author Victor Julien <victor@inliniac.net>
  *
- * Tracks chunk based file transfers. Chunks may be transfered out
- * of order, but cannot be transfered in parallel. So only one
+ * Tracks chunk based file transfers. Chunks may be transferred out
+ * of order, but cannot be transferred in parallel. So only one
  * chunk at a time.
  *
  * GAP handling. If a data gap is encountered, the file is truncated
@@ -49,16 +49,21 @@ impl FileChunk {
 }
 
 #[derive(Debug)]
+#[derive(Default)]
 pub struct FileTransferTracker {
     pub tracked: u64,
     cur_ooo: u64,   // how many bytes do we have queued from ooo chunks
     track_id: u32,
     chunk_left: u32,
 
+    pub file: FileContainer,
+    pub file_flags: u16,
+
     pub tx_id: u64,
 
     fill_bytes: u8,
     pub file_open: bool,
+    file_closed: bool,
     chunk_is_last: bool,
     chunk_is_ooo: bool,
     file_is_truncated: bool,
@@ -72,52 +77,46 @@ pub struct FileTransferTracker {
 impl FileTransferTracker {
     pub fn new() -> FileTransferTracker {
         FileTransferTracker {
-            tracked:0,
-            cur_ooo:0,
-            track_id:0,
-            chunk_left:0,
-            tx_id:0,
-            fill_bytes:0,
-            file_open:false,
-            chunk_is_last:false,
-            chunk_is_ooo:false,
-            file_is_truncated:false,
-            cur_ooo_chunk_offset:0,
             chunks:HashMap::new(),
-            in_flight: 0,
+            ..Default::default()
         }
     }
 
     pub fn is_done(&self) -> bool {
-        self.file_open == false
+        !self.file_open
     }
 
-    fn open(&mut self, config: &'static SuricataFileContext,
-            files: &mut FileContainer, flags: u16, name: &[u8]) -> i32
+    pub fn is_initialized(&self) -> bool {
+        return self.file_open || self.file_is_truncated || self.file_closed;
+    }
+
+    fn open(&mut self, config: &'static SuricataFileContext, name: &[u8]) -> i32
     {
-        let r = files.file_open(config, self.track_id, name, flags);
+        let r = self.file.file_open(config, self.track_id, name, self.file_flags);
         if r == 0 {
-            files.file_set_txid_on_last_file(self.tx_id);
             self.file_open = true;
         }
         r
     }
 
-    pub fn close(&mut self, files: &mut FileContainer, flags: u16) {
+    pub fn close(&mut self, config: &'static SuricataFileContext)
+    {
         if !self.file_is_truncated {
             SCLogDebug!("closing file with id {}", self.track_id);
-            files.file_close(&self.track_id, flags);
+            self.file.file_close(config, &self.track_id, self.file_flags);
         }
         self.file_open = false;
+        self.file_closed = true;
         self.tracked = 0;
     }
 
-    pub fn trunc (&mut self, files: &mut FileContainer, flags: u16) {
+    pub fn trunc (&mut self, config: &'static SuricataFileContext)
+    {
         if self.file_is_truncated || !self.file_open {
             return;
         }
-        let myflags = flags | 1; // TODO util-file.c::FILE_TRUNCATED
-        files.file_close(&self.track_id, myflags);
+        let myflags = self.file_flags | 1; // TODO util-file.c::FILE_TRUNCATED
+        self.file.file_close(config, &self.track_id, myflags);
         SCLogDebug!("truncated file");
         self.file_is_truncated = true;
         self.chunks.clear();
@@ -125,20 +124,13 @@ impl FileTransferTracker {
         self.cur_ooo = 0;
     }
 
-    pub fn create(&mut self, _name: &[u8], _file_size: u64) {
-        if self.file_open == true { panic!("close existing file first"); }
-
-        SCLogDebug!("CREATE: name {:?} file_size {}", _name, _file_size);
-    }
-
     pub fn new_chunk(&mut self, config: &'static SuricataFileContext,
-            files: &mut FileContainer, flags: u16,
             name: &[u8], data: &[u8], chunk_offset: u64, chunk_size: u32,
             fill_bytes: u8, is_last: bool, xid: &u32) -> u32
     {
         if self.chunk_left != 0 || self.fill_bytes != 0 {
             SCLogDebug!("current chunk incomplete: truncating");
-            self.trunc(files, flags);
+            self.trunc(config);
         }
 
         SCLogDebug!("NEW CHUNK: chunk_size {} fill_bytes {}", chunk_size, fill_bytes);
@@ -150,7 +142,7 @@ impl FileTransferTracker {
             SCLogDebug!("NEW CHUNK IS OOO: expected {}, got {}", self.tracked, chunk_offset);
             if is_last {
                 SCLogDebug!("last chunk is out of order, this means we missed data before");
-                self.trunc(files, flags);
+                self.trunc(config);
             }
             self.chunk_is_ooo = true;
             self.cur_ooo_chunk_offset = chunk_offset;
@@ -160,17 +152,17 @@ impl FileTransferTracker {
         self.fill_bytes = fill_bytes;
         self.chunk_is_last = is_last;
 
-        if self.file_is_truncated {
+        if self.file_is_truncated || self.file_closed {
             return 0;
         }
-        if self.file_open == false {
+        if !self.file_open {
             SCLogDebug!("NEW CHUNK: FILE OPEN");
             self.track_id = *xid;
-            self.open(config, files, flags, name);
+            self.open(config, name);
         }
 
-        if self.file_open == true {
-            let res = self.update(files, flags, data, 0);
+        if self.file_open {
+            let res = self.update(config, data, 0);
             SCLogDebug!("NEW CHUNK: update res {:?}", res);
             return res;
         }
@@ -181,13 +173,14 @@ impl FileTransferTracker {
     /// update the file tracker
     /// If gap_size > 0 'data' should not be used.
     /// return how much we consumed of data
-    pub fn update(&mut self, files: &mut FileContainer, flags: u16, data: &[u8], gap_size: u32) -> u32 {
+    pub fn update(&mut self, config: &'static SuricataFileContext, data: &[u8], gap_size: u32) -> u32
+    {
         if self.file_is_truncated {
             let consumed = std::cmp::min(data.len() as u32, self.chunk_left);
             self.chunk_left = self.chunk_left.saturating_sub(data.len() as u32);
             return consumed;
         }
-        let mut consumed = 0 as usize;
+        let mut consumed = 0_usize;
         let is_gap = gap_size > 0;
         if is_gap || gap_size > 0 {
             SCLogDebug!("is_gap {} size {} ooo? {}", is_gap, gap_size, self.chunk_is_ooo);
@@ -195,9 +188,9 @@ impl FileTransferTracker {
 
         if self.chunk_left == 0 && self.fill_bytes == 0 {
             //SCLogDebug!("UPDATE: nothing to do");
-            if self.chunk_is_last == true {
+            if self.chunk_is_last {
                 SCLogDebug!("last empty chunk, closing");
-                self.close(files, flags);
+                self.close(config);
                 self.chunk_is_last = false;
             }
             return 0
@@ -221,15 +214,15 @@ impl FileTransferTracker {
             if self.chunk_left <= data.len() as u32 {
                 let d = &data[0..self.chunk_left as usize];
 
-                if self.chunk_is_ooo == false {
-                    let res = files.file_append(&self.track_id, d, is_gap);
+                if !self.chunk_is_ooo {
+                    let res = self.file.file_append(config, &self.track_id, d, is_gap);
                     match res {
                         0   => { },
                         -2  => {
                             self.file_is_truncated = true;
                         },
                         _ => {
-                            SCLogDebug!("got error so truncing file");
+                            SCLogDebug!("got error so truncating file");
                             self.file_is_truncated = true;
                         },
                     }
@@ -268,21 +261,21 @@ impl FileTransferTracker {
                 } else {
                     self.chunk_left = 0;
 
-                    if self.chunk_is_ooo == false {
+                    if !self.chunk_is_ooo {
                         loop {
                             let _offset = self.tracked;
                             match self.chunks.remove(&self.tracked) {
                                 Some(c) => {
                                     self.in_flight -= c.chunk.len() as u64;
 
-                                    let res = files.file_append(&self.track_id, &c.chunk, c.contains_gap);
+                                    let res = self.file.file_append(config, &self.track_id, &c.chunk, c.contains_gap);
                                     match res {
                                         0   => { },
                                         -2  => {
                                             self.file_is_truncated = true;
                                         },
                                         _ => {
-                                            SCLogDebug!("got error so truncing file");
+                                            SCLogDebug!("got error so truncating file");
                                             self.file_is_truncated = true;
                                         },
                                     }
@@ -305,24 +298,24 @@ impl FileTransferTracker {
                         self.cur_ooo_chunk_offset = 0;
                     }
                 }
-                if self.chunk_is_last == true {
+                if self.chunk_is_last {
                     SCLogDebug!("last chunk, closing");
-                    self.close(files, flags);
+                    self.close(config);
                     self.chunk_is_last = false;
                 } else {
                     SCLogDebug!("NOT last chunk, keep going");
                 }
 
             } else {
-                if self.chunk_is_ooo == false {
-                    let res = files.file_append(&self.track_id, data, is_gap);
+                if !self.chunk_is_ooo {
+                    let res = self.file.file_append(config, &self.track_id, data, is_gap);
                     match res {
                         0   => { },
                         -2  => {
                             self.file_is_truncated = true;
                         },
                         _ => {
-                            SCLogDebug!("got error so truncing file");
+                            SCLogDebug!("got error so truncating file");
                             self.file_is_truncated = true;
                         },
                     }
