@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2023 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -41,6 +41,7 @@
 #include "util-unittest.h"
 #include "util-debug.h"
 #include "util-path.h"
+#include "util-conf.h"
 
 /** Maximum size of a complete domain name. */
 #define NODE_NAME_MAX 1024
@@ -55,23 +56,22 @@ static ConfNode *root_backup = NULL;
  * This function exits on memory failure as creating configuration
  * nodes is usually part of application initialization.
  *
+ * \param parent The node to use as the parent
  * \param name The name of the configuration node to get.
  * \param final Flag to set created nodes as final or not.
  *
  * \retval The existing configuration node if it exists, or a newly
  *   created node for the provided name.  On error, NULL will be returned.
  */
-static ConfNode *ConfGetNodeOrCreate(const char *name, int final)
+ConfNode *ConfNodeGetNodeOrCreate(ConfNode *parent, const char *name, int final)
 {
-    ConfNode *parent = root;
     ConfNode *node = NULL;
     char node_name[NODE_NAME_MAX];
     char *key;
     char *next;
 
     if (strlcpy(node_name, name, sizeof(node_name)) >= sizeof(node_name)) {
-        SCLogError(SC_ERR_CONF_NAME_TOO_LONG,
-            "Configuration name too long: %s", name);
+        SCLogError("Configuration name too long: %s", name);
         return NULL;
     }
 
@@ -83,16 +83,14 @@ static ConfNode *ConfGetNodeOrCreate(const char *name, int final)
         if ((node = ConfNodeLookupChild(parent, key)) == NULL) {
             node = ConfNodeNew();
             if (unlikely(node == NULL)) {
-                SCLogWarning(SC_ERR_MEM_ALLOC,
-                    "Failed to allocate memory for configuration.");
+                SCLogWarning("Failed to allocate memory for configuration.");
                 goto end;
             }
             node->name = SCStrdup(key);
             if (unlikely(node->name == NULL)) {
                 ConfNodeFree(node);
                 node = NULL;
-                SCLogWarning(SC_ERR_MEM_ALLOC,
-                    "Failed to allocate memory for configuration.");
+                SCLogWarning("Failed to allocate memory for configuration.");
                 goto end;
             }
             node->parent = parent;
@@ -107,6 +105,15 @@ end:
     return node;
 }
 
+/**
+ * \brief Wrapper function for ConfNodeGetNodeOrCreate that operates
+ *     on the current root node.
+ */
+static ConfNode *ConfGetNodeOrCreate(const char *name, int final)
+{
+    return ConfNodeGetNodeOrCreate(root, name, final);
+}
+
 /**
  * \brief Initialize the configuration system.
  */
@@ -118,9 +125,8 @@ void ConfInit(void)
     }
     root = ConfNodeNew();
     if (root == NULL) {
-            FatalError(SC_ERR_FATAL,
-                       "ERROR: Failed to allocate memory for root configuration node, "
-                       "aborting.");
+        FatalError("ERROR: Failed to allocate memory for root configuration node, "
+                   "aborting.");
     }
     SCLogDebug("configuration module initialized");
 }
@@ -180,8 +186,7 @@ ConfNode *ConfGetNode(const char *name)
     char *next;
 
     if (strlcpy(node_name, name, sizeof(node_name)) >= sizeof(node_name)) {
-        SCLogError(SC_ERR_CONF_NAME_TOO_LONG,
-            "Configuration name too long: %s", name);
+        SCLogError("Configuration name too long: %s", name);
         return NULL;
     }
 
@@ -340,50 +345,6 @@ int ConfGet(const char *name, const char **vptr)
     }
 }
 
-/**
- * \brief Retrieve the value of a configuration node.
- *
- * This function will return the value for a configuration node based
- * on the full name of the node. This function notifies if vptr returns NULL
- * or if name is set to NULL.
- *
- * \param name Name of configuration parameter to get.
- * \param vptr Pointer that will be set to the configuration value parameter.
- *   Note that this is just a reference to the actual value, not a copy.
- *
- * \retval 0 will be returned if name was not found,
- *    1 will be returned if the name and it's value was found,
- *   -1 if the value returns NULL,
- *   -2 if name is NULL.
- */
-int ConfGetValue(const char *name, const char **vptr)
-{
-    ConfNode *node;
-
-    if (name == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT,"parameter 'name' is NULL");
-        return -2;
-    }
-
-    node = ConfGetNode(name);
-
-    if (node == NULL) {
-        SCLogDebug("failed to lookup configuration parameter '%s'", name);
-        return 0;
-    }
-    else {
-
-        if (node->val == NULL) {
-            SCLogDebug("value for configuration parameter '%s' is NULL", name);
-            return -1;
-        }
-
-        *vptr = node->val;
-        return 1;
-    }
-
-}
-
 int ConfGetChildValue(const ConfNode *base, const char *name, const char **vptr)
 {
     ConfNode *node = ConfNodeLookupChild(base, name);
@@ -431,7 +392,7 @@ int ConfGetChildValueWithDefault(const ConfNode *base, const ConfNode *dflt,
  * configuration value.
  *
  * \retval 1 will be returned if the name is found and was properly
- * converted to an interger, otherwise 0 will be returned.
+ * converted to an integer, otherwise 0 will be returned.
  */
 int ConfGetInt(const char *name, intmax_t *val)
 {
@@ -443,21 +404,24 @@ int ConfGetInt(const char *name, intmax_t *val)
         return 0;
 
     if (strval == NULL) {
-        SCLogError(SC_ERR_INVALID_YAML_CONF_ENTRY, "malformed integer value "
-                "for %s: NULL", name);
+        SCLogError("malformed integer value "
+                   "for %s: NULL",
+                name);
         return 0;
     }
 
     errno = 0;
     tmpint = strtoimax(strval, &endptr, 0);
     if (strval[0] == '\0' || *endptr != '\0') {
-        SCLogError(SC_ERR_INVALID_YAML_CONF_ENTRY, "malformed integer value "
-                "for %s: '%s'", name, strval);
+        SCLogError("malformed integer value "
+                   "for %s: '%s'",
+                name, strval);
         return 0;
     }
     if (errno == ERANGE && (tmpint == INTMAX_MAX || tmpint == INTMAX_MIN)) {
-        SCLogError(SC_ERR_INVALID_YAML_CONF_ENTRY, "integer value for %s out "
-                "of range: '%s'", name, strval);
+        SCLogError("integer value for %s out "
+                   "of range: '%s'",
+                name, strval);
         return 0;
     }
 
@@ -476,13 +440,15 @@ int ConfGetChildValueInt(const ConfNode *base, const char *name, intmax_t *val)
     errno = 0;
     tmpint = strtoimax(strval, &endptr, 0);
     if (strval[0] == '\0' || *endptr != '\0') {
-        SCLogError(SC_ERR_INVALID_YAML_CONF_ENTRY, "malformed integer value "
-                "for %s with base %s: '%s'", name, base->name, strval);
+        SCLogError("malformed integer value "
+                   "for %s with base %s: '%s'",
+                name, base->name, strval);
         return 0;
     }
     if (errno == ERANGE && (tmpint == INTMAX_MAX || tmpint == INTMAX_MIN)) {
-        SCLogError(SC_ERR_INVALID_YAML_CONF_ENTRY, "integer value for %s with "
-                " base %s out of range: '%s'", name, base->name, strval);
+        SCLogError("integer value for %s with "
+                   " base %s out of range: '%s'",
+                name, base->name, strval);
         return 0;
     }
 
@@ -502,9 +468,8 @@ int ConfGetChildValueIntWithDefault(const ConfNode *base, const ConfNode *dflt,
     return ret;
 }
 
-
 /**
- * \brief Retrieve a configuration value as an boolen.
+ * \brief Retrieve a configuration value as a boolean.
  *
  * \param name Name of configuration parameter to get.
  * \param val Pointer to an int that will be set to 1 for true, or 0
@@ -518,7 +483,7 @@ int ConfGetBool(const char *name, int *val)
     const char *strval = NULL;
 
     *val = 0;
-    if (ConfGetValue(name, &strval) != 1)
+    if (ConfGet(name, &strval) != 1)
         return 0;
 
     *val = ConfValIsTrue(strval);
@@ -526,6 +491,11 @@ int ConfGetBool(const char *name, int *val)
     return 1;
 }
 
+/**
+ * Get a boolean value from the provided ConfNode.
+ *
+ * \retval 1 If the value exists, 0 if not.
+ */
 int ConfGetChildValueBool(const ConfNode *base, const char *name, int *val)
 {
     const char *strval = NULL;
@@ -969,13 +939,67 @@ void ConfNodePrune(ConfNode *node)
  *
  * \param node the node to check.
  *
- * \return 1 if node is a seuence, otherwise 0.
+ * \return 1 if node is a sequence, otherwise 0.
  */
 int ConfNodeIsSequence(const ConfNode *node)
 {
     return node->is_seq == 0 ? 0 : 1;
 }
 
+/**
+ * @brief Finds an interface from the list of interfaces.
+ * @param ifaces_node_name - name of the node which holds a list of interfaces
+ * @param iface - interfaces name
+ * @return NULL on failure otherwise a valid pointer
+ */
+ConfNode *ConfSetIfaceNode(const char *ifaces_node_name, const char *iface)
+{
+    ConfNode *if_node;
+    ConfNode *ifaces_list_node;
+    /* Find initial node which holds all interfaces */
+    ifaces_list_node = ConfGetNode(ifaces_node_name);
+    if (ifaces_list_node == NULL) {
+        SCLogError("unable to find %s config", ifaces_node_name);
+        return NULL;
+    }
+
+    if_node = ConfFindDeviceConfig(ifaces_list_node, iface);
+    if (if_node == NULL)
+        SCLogNotice("unable to find interface %s in DPDK config", iface);
+
+    return if_node;
+}
+
+/**
+ * @brief Finds and sets root and default node of the interface.
+ * @param ifaces_node_name Node which holds list of interfaces
+ * @param iface Name of the interface e.g. eth3
+ * @param if_root Node which will hold the interface configuration
+ * @param if_default Node which is the default configuration in the given list of interfaces
+ * @return 0 on success, -ENODEV when neither the root interface nor the default interface was found
+ */
+int ConfSetRootAndDefaultNodes(
+        const char *ifaces_node_name, const char *iface, ConfNode **if_root, ConfNode **if_default)
+{
+    const char *default_iface = "default";
+    *if_root = ConfSetIfaceNode(ifaces_node_name, iface);
+    *if_default = ConfSetIfaceNode(ifaces_node_name, default_iface);
+
+    if (*if_root == NULL && *if_default == NULL) {
+        SCLogError("unable to find configuration for the interface \"%s\" or the default "
+                   "configuration (\"%s\")",
+                iface, default_iface);
+        return (-ENODEV);
+    }
+
+    /* If there is no setting for current interface use default one as main iface */
+    if (*if_root == NULL) {
+        *if_root = *if_default;
+        *if_default = NULL;
+    }
+    return 0;
+}
+
 #ifdef UNITTESTS
 
 /**
@@ -1034,7 +1058,7 @@ static int ConfTestOverrideValue1(void)
 }
 
 /**
- * Test that a final value will not be overrided by a ConfSet.
+ * Test that a final value will not be overridden by a ConfSet.
  */
 static int ConfTestOverrideValue2(void)
 {
