@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2022 Open Information Security Foundation
+/* Copyright (C) 2007-2023 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -1021,6 +1021,11 @@ typedef struct DetectEngineCtx_ {
     pcre2_code *reference_conf_regex;
     pcre2_match_data *reference_conf_regex_match;
 
+    /* --engine-analysis */
+    struct EngineAnalysisCtx_ *ea;
+
+    /* path to the tenant yaml for this engine */
+    char *tenant_path;
 } DetectEngineCtx;
 
 /* Engine groups profiles (low, medium, high, custom) */
