@@ -242,7 +242,7 @@ static void *TmThreadsSlotPktAcqLoop(void *td)
 
     /* Drop the capabilities for this thread */
     SCDropCaps(tv);
-
+    CaptureStatsSetup(tv);
     PacketPoolInit();
 
     /* check if we are setup properly */
@@ -372,6 +372,7 @@ static void *TmThreadsSlotVar(void *td)
     char run = 1;
     TmEcode r = TM_ECODE_OK;
 
+    CaptureStatsSetup(tv);
     PacketPoolInit();//Empty();
 
     SCSetThreadName(tv->name);
@@ -1670,7 +1671,8 @@ TmEcode TmThreadSpawn(ThreadVars *tv)
 
     int rc = pthread_create(&tv->t, &attr, tv->tm_func, (void *)tv);
     if (rc) {
-        FatalError("Unable to create thread with pthread_create() is %" PRId32, rc);
+        FatalError("Unable to create thread with pthread_create(): retval %d: %s", rc,
+                strerror(errno));
     }
 
 #if DEBUG && HAVE_PTHREAD_GETATTR_NP
