@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2012 Open Information Security Foundation
+/* Copyright (C) 2007-2023 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -27,20 +27,15 @@
  */
 
 #include "suricata-common.h"
-#include "decode.h"
-#include "detect.h"
-#include "detect-engine-prefilter.h"
-#include "conf.h"
-#include "flow-worker.h"
-
-#include "tm-threads.h"
-
-#include "util-unittest.h"
-#include "util-byte.h"
 #include "util-profiling.h"
-#include "util-profiling-locks.h"
 
 #ifdef PROFILING
+#include "tm-threads.h"
+#include "conf.h"
+#include "util-unittest.h"
+#include "util-byte.h"
+#include "util-profiling-locks.h"
+#include "util-conf.h"
 
 #ifndef MIN
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
@@ -118,7 +113,7 @@ thread_local int profiling_rules_entered = 0;
 
 void SCProfilingDumpPacketStats(void);
 const char * PacketProfileDetectIdToString(PacketProfileDetectId id);
-const char * PacketProfileLoggertIdToString(LoggerId id);
+const char *PacketProfileLoggerIdToString(LoggerId id);
 static void PrintCSVHeader(void);
 
 static void FormatNumber(uint64_t num, char *str, size_t size)
@@ -159,8 +154,7 @@ SCProfilingInit(void)
             profiling_packets_enabled = 1;
 
             if (pthread_mutex_init(&packet_profile_lock, NULL) != 0) {
-                        FatalError(SC_ERR_FATAL,
-                                   "Failed to initialize packet profiling mutex.");
+                FatalError("Failed to initialize packet profiling mutex.");
             }
             memset(&packet_profile_data4, 0, sizeof(packet_profile_data4));
             memset(&packet_profile_data6, 0, sizeof(packet_profile_data6));
@@ -207,7 +201,7 @@ SCProfilingInit(void)
 
                 profiling_csv_file_name = SCMalloc(PATH_MAX);
                 if (unlikely(profiling_csv_file_name == NULL)) {
-                    FatalError(SC_ERR_FATAL, "out of memory");
+                    FatalError("out of memory");
                 }
                 snprintf(profiling_csv_file_name, PATH_MAX, "%s/%s", log_dir, filename);
 
@@ -229,7 +223,8 @@ SCProfilingInit(void)
     if (conf != NULL) {
         if (ConfNodeChildValueIsTrue(conf, "enabled")) {
 #ifndef PROFILE_LOCKING
-            SCLogWarning(SC_WARN_PROFILE, "lock profiling not compiled in. Add --enable-profiling-locks to configure.");
+            SCLogWarning(
+                    "lock profiling not compiled in. Add --enable-profiling-locks to configure.");
 #else
             profiling_locks_enabled = 1;
 
@@ -241,7 +236,7 @@ SCProfilingInit(void)
 
                 profiling_locks_file_name = SCMalloc(PATH_MAX);
                 if (unlikely(profiling_locks_file_name == NULL)) {
-                    FatalError(SC_ERR_FATAL, "can't duplicate file name");
+                    FatalError("can't duplicate file name");
                 }
 
                 snprintf(profiling_locks_file_name, PATH_MAX, "%s/%s", log_dir, filename);
@@ -348,7 +343,6 @@ static void DumpFlowWorker(FILE *fp)
 
 void SCProfilingDumpPacketStats(void)
 {
-    int i;
     FILE *fp;
     char totalstr[256];
     uint64_t total;
@@ -360,8 +354,7 @@ void SCProfilingDumpPacketStats(void)
         fp = fopen(profiling_packets_file_name, profiling_packets_file_mode);
 
         if (fp == NULL) {
-            SCLogError(SC_ERR_FOPEN, "failed to open %s: %s",
-                    profiling_packets_file_name, strerror(errno));
+            SCLogError("failed to open %s: %s", profiling_packets_file_name, strerror(errno));
             return;
         }
     } else {
@@ -375,16 +368,15 @@ void SCProfilingDumpPacketStats(void)
     fprintf(fp, "%-6s   %-5s   %-12s   %-12s   %-12s   %-12s   %-12s  %-3s\n",
             "------", "-----", "----------", "------------", "------------", "-----------", "-----------", "---");
     total = 0;
-    for (i = 0; i < 257; i++) {
+    for (int i = 0; i < 257; i++) {
         SCProfilePacketData *pd = &packet_profile_data4[i];
         total += pd->tot;
         pd = &packet_profile_data6[i];
         total += pd->tot;
     }
 
-    for (i = 0; i < 257; i++) {
+    for (int i = 0; i < 257; i++) {
         SCProfilePacketData *pd = &packet_profile_data4[i];
-
         if (pd->cnt == 0) {
             continue;
         }
@@ -397,9 +389,8 @@ void SCProfilingDumpPacketStats(void)
             pd->min, pd->max, (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
     }
 
-    for (i = 0; i < 257; i++) {
+    for (int i = 0; i < 257; i++) {
         SCProfilePacketData *pd = &packet_profile_data6[i];
-
         if (pd->cnt == 0) {
             continue;
         }
@@ -431,14 +422,12 @@ void SCProfilingDumpPacketStats(void)
 #else
     fprintf(fp, "\n");
 #endif
-    int m;
     total = 0;
-    for (m = 0; m < TMM_SIZE; m++) {
+    for (int m = 0; m < TMM_SIZE; m++) {
         if (tmm_modules[m].flags & TM_FLAG_LOGAPI_TM)
             continue;
 
-        int p;
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_tmm_data4[m][p];
             total += pd->tot;
 
@@ -447,14 +436,12 @@ void SCProfilingDumpPacketStats(void)
         }
     }
 
-    for (m = 0; m < TMM_SIZE; m++) {
+    for (int m = 0; m < TMM_SIZE; m++) {
         if (tmm_modules[m].flags & TM_FLAG_LOGAPI_TM)
             continue;
 
-        int p;
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_tmm_data4[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -474,14 +461,12 @@ void SCProfilingDumpPacketStats(void)
         }
     }
 
-    for (m = 0; m < TMM_SIZE; m++) {
+    for (int m = 0; m < TMM_SIZE; m++) {
         if (tmm_modules[m].flags & TM_FLAG_LOGAPI_TM)
             continue;
 
-        int p;
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_tmm_data6[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -505,21 +490,18 @@ void SCProfilingDumpPacketStats(void)
             "--------------------", "------", "-----", "----------", "------------", "------------", "-----------");
 
     total = 0;
-    for (m = 0; m < ALPROTO_MAX; m++) {
-        int p;
-        for (p = 0; p < 257; p++) {
-            SCProfilePacketData *pd = &packet_profile_app_data4[m][p];
+    for (AppProto a = 0; a < ALPROTO_MAX; a++) {
+        for (int p = 0; p < 257; p++) {
+            SCProfilePacketData *pd = &packet_profile_app_data4[a][p];
             total += pd->tot;
 
-            pd = &packet_profile_app_data6[m][p];
+            pd = &packet_profile_app_data6[a][p];
             total += pd->tot;
         }
     }
-    for (m = 0; m < ALPROTO_MAX; m++) {
-        int p;
-        for (p = 0; p < 257; p++) {
-            SCProfilePacketData *pd = &packet_profile_app_data4[m][p];
-
+    for (AppProto a = 0; a < ALPROTO_MAX; a++) {
+        for (int p = 0; p < 257; p++) {
+            SCProfilePacketData *pd = &packet_profile_app_data4[a][p];
             if (pd->cnt == 0) {
                 continue;
             }
@@ -528,16 +510,17 @@ void SCProfilingDumpPacketStats(void)
             double percent = (long double)pd->tot /
                 (long double)total * 100;
 
-            fprintf(fp, "%-20s    IPv4     %3d  %12"PRIu64"     %12"PRIu64"   %12"PRIu64"  %12"PRIu64"  %12s  %-6.2f\n",
-                    AppProtoToString(m), p, pd->cnt, pd->min, pd->max, (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
+            fprintf(fp,
+                    "%-20s    IPv4     %3d  %12" PRIu64 "     %12" PRIu64 "   %12" PRIu64
+                    "  %12" PRIu64 "  %12s  %-6.2f\n",
+                    AppProtoToString(a), p, pd->cnt, pd->min, pd->max,
+                    (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
         }
     }
 
-    for (m = 0; m < ALPROTO_MAX; m++) {
-        int p;
-        for (p = 0; p < 257; p++) {
-            SCProfilePacketData *pd = &packet_profile_app_data6[m][p];
-
+    for (AppProto a = 0; a < ALPROTO_MAX; a++) {
+        for (int p = 0; p < 257; p++) {
+            SCProfilePacketData *pd = &packet_profile_app_data6[a][p];
             if (pd->cnt == 0) {
                 continue;
             }
@@ -546,17 +529,18 @@ void SCProfilingDumpPacketStats(void)
             double percent = (long double)pd->tot /
                 (long double)total * 100;
 
-            fprintf(fp, "%-20s    IPv6     %3d  %12"PRIu64"     %12"PRIu64"   %12"PRIu64"  %12"PRIu64"  %12s  %-6.2f\n",
-                    AppProtoToString(m), p, pd->cnt, pd->min, pd->max, (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
+            fprintf(fp,
+                    "%-20s    IPv6     %3d  %12" PRIu64 "     %12" PRIu64 "   %12" PRIu64
+                    "  %12" PRIu64 "  %12s  %-6.2f\n",
+                    AppProtoToString(a), p, pd->cnt, pd->min, pd->max,
+                    (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
         }
     }
 
     /* proto detect output */
     {
-        int p;
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_app_pd_data4[p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -566,9 +550,8 @@ void SCProfilingDumpPacketStats(void)
                     "Proto detect", p, pd->cnt, pd->min, pd->max, (uint64_t)(pd->tot / pd->cnt), totalstr);
         }
 
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_app_pd_data6[p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -580,9 +563,8 @@ void SCProfilingDumpPacketStats(void)
     }
 
     total = 0;
-    for (m = 0; m < PROF_DETECT_SIZE; m++) {
-        int p;
-        for (p = 0; p < 257; p++) {
+    for (int m = 0; m < PROF_DETECT_SIZE; m++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_detect_data4[m][p];
             total += pd->tot;
 
@@ -591,7 +573,6 @@ void SCProfilingDumpPacketStats(void)
         }
     }
 
-
     fprintf(fp, "\n%-24s   %-6s   %-5s   %-12s   %-12s   %-12s   %-12s   %-12s  %-3s",
             "Log Thread Module", "IP ver", "Proto", "cnt", "min", "max", "avg", "tot", "%%");
 #ifdef PROFILE_LOCKING
@@ -609,12 +590,11 @@ void SCProfilingDumpPacketStats(void)
     fprintf(fp, "\n");
 #endif
     total = 0;
-    for (m = 0; m < TMM_SIZE; m++) {
+    for (int m = 0; m < TMM_SIZE; m++) {
         if (!(tmm_modules[m].flags & TM_FLAG_LOGAPI_TM))
             continue;
 
-        int p;
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_tmm_data4[m][p];
             total += pd->tot;
 
@@ -623,14 +603,12 @@ void SCProfilingDumpPacketStats(void)
         }
     }
 
-    for (m = 0; m < TMM_SIZE; m++) {
+    for (int m = 0; m < TMM_SIZE; m++) {
         if (!(tmm_modules[m].flags & TM_FLAG_LOGAPI_TM))
             continue;
 
-        int p;
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_tmm_data4[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -650,14 +628,12 @@ void SCProfilingDumpPacketStats(void)
         }
     }
 
-    for (m = 0; m < TMM_SIZE; m++) {
+    for (int m = 0; m < TMM_SIZE; m++) {
         if (!(tmm_modules[m].flags & TM_FLAG_LOGAPI_TM))
             continue;
 
-        int p;
-        for (p = 0; p < 257; p++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_tmm_data6[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -674,9 +650,8 @@ void SCProfilingDumpPacketStats(void)
     fprintf(fp, "\nLogger/output stats:\n");
 
     total = 0;
-    for (m = 0; m < LOGGER_SIZE; m++) {
-        int p;
-        for (p = 0; p < 256; p++) {
+    for (int m = 0; m < LOGGER_SIZE; m++) {
+        for (int p = 0; p < 256; p++) {
             SCProfilePacketData *pd = &packet_profile_log_data4[m][p];
             total += pd->tot;
             pd = &packet_profile_log_data6[m][p];
@@ -688,11 +663,9 @@ void SCProfilingDumpPacketStats(void)
             "Logger", "IP ver", "Proto", "cnt", "min", "max", "avg", "tot");
     fprintf(fp, "%-24s   %-6s   %-5s   %-12s   %-12s   %-12s   %-12s   %-12s\n",
             "------------------------", "------", "-----", "----------", "------------", "------------", "-----------", "-----------");
-    for (m = 0; m < LOGGER_SIZE; m++) {
-        int p;
-        for (p = 0; p < 256; p++) {
+    for (int m = 0; m < LOGGER_SIZE; m++) {
+        for (int p = 0; p < 256; p++) {
             SCProfilePacketData *pd = &packet_profile_log_data4[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -701,15 +674,16 @@ void SCProfilingDumpPacketStats(void)
             double percent = (long double)pd->tot /
                 (long double)total * 100;
 
-            fprintf(fp, "%-24s    IPv4     %3d  %12"PRIu64"     %12"PRIu64"   %12"PRIu64"  %12"PRIu64"  %12s  %-6.2f\n",
-                    PacketProfileLoggertIdToString(m), p, pd->cnt, pd->min, pd->max, (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
+            fprintf(fp,
+                    "%-24s    IPv4     %3d  %12" PRIu64 "     %12" PRIu64 "   %12" PRIu64
+                    "  %12" PRIu64 "  %12s  %-6.2f\n",
+                    PacketProfileLoggerIdToString(m), p, pd->cnt, pd->min, pd->max,
+                    (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
         }
     }
-    for (m = 0; m < LOGGER_SIZE; m++) {
-        int p;
-        for (p = 0; p < 256; p++) {
+    for (int m = 0; m < LOGGER_SIZE; m++) {
+        for (int p = 0; p < 256; p++) {
             SCProfilePacketData *pd = &packet_profile_log_data6[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -718,17 +692,19 @@ void SCProfilingDumpPacketStats(void)
             double percent = (long double)pd->tot /
                 (long double)total * 100;
 
-            fprintf(fp, "%-24s    IPv6     %3d  %12"PRIu64"     %12"PRIu64"   %12"PRIu64"  %12"PRIu64"  %12s  %-6.2f\n",
-                    PacketProfileLoggertIdToString(m), p, pd->cnt, pd->min, pd->max, (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
+            fprintf(fp,
+                    "%-24s    IPv6     %3d  %12" PRIu64 "     %12" PRIu64 "   %12" PRIu64
+                    "  %12" PRIu64 "  %12s  %-6.2f\n",
+                    PacketProfileLoggerIdToString(m), p, pd->cnt, pd->min, pd->max,
+                    (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
         }
     }
 
     fprintf(fp, "\nGeneral detection engine stats:\n");
 
     total = 0;
-    for (m = 0; m < PROF_DETECT_SIZE; m++) {
-        int p;
-        for (p = 0; p < 257; p++) {
+    for (int m = 0; m < PROF_DETECT_SIZE; m++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_detect_data4[m][p];
             total += pd->tot;
             pd = &packet_profile_detect_data6[m][p];
@@ -740,11 +716,9 @@ void SCProfilingDumpPacketStats(void)
             "Detection phase", "IP ver", "Proto", "cnt", "min", "max", "avg", "tot");
     fprintf(fp, "%-24s   %-6s   %-5s   %-12s   %-12s   %-12s   %-12s   %-12s\n",
             "------------------------", "------", "-----", "----------", "------------", "------------", "-----------", "-----------");
-    for (m = 0; m < PROF_DETECT_SIZE; m++) {
-        int p;
-        for (p = 0; p < 257; p++) {
+    for (int m = 0; m < PROF_DETECT_SIZE; m++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_detect_data4[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -757,11 +731,9 @@ void SCProfilingDumpPacketStats(void)
                     PacketProfileDetectIdToString(m), p, pd->cnt, pd->min, pd->max, (uint64_t)(pd->tot / pd->cnt), totalstr, percent);
         }
     }
-    for (m = 0; m < PROF_DETECT_SIZE; m++) {
-        int p;
-        for (p = 0; p < 257; p++) {
+    for (int m = 0; m < PROF_DETECT_SIZE; m++) {
+        for (int p = 0; p < 257; p++) {
             SCProfilePacketData *pd = &packet_profile_detect_data6[m][p];
-
             if (pd->cnt == 0) {
                 continue;
             }
@@ -795,7 +767,7 @@ static void PrintCSVHeader(void)
 
     /* individual loggers */
     for (LoggerId i = 0; i < LOGGER_SIZE; i++) {
-        fprintf(packet_profile_csv_fp, "%s,", PacketProfileLoggertIdToString(i));
+        fprintf(packet_profile_csv_fp, "%s,", PacketProfileLoggerIdToString(i));
     }
 
     fprintf(packet_profile_csv_fp, "\n");
@@ -1260,20 +1232,21 @@ int SCProfileRuleStart(Packet *p)
 const char * PacketProfileDetectIdToString(PacketProfileDetectId id)
 {
     switch (id) {
-        CASE_CODE (PROF_DETECT_SETUP);
-        CASE_CODE (PROF_DETECT_GETSGH);
-        CASE_CODE (PROF_DETECT_IPONLY);
-        CASE_CODE (PROF_DETECT_RULES);
-        CASE_CODE (PROF_DETECT_PF_PKT);
-        CASE_CODE (PROF_DETECT_PF_PAYLOAD);
-        CASE_CODE (PROF_DETECT_PF_TX);
-        CASE_CODE (PROF_DETECT_PF_SORT1);
-        CASE_CODE (PROF_DETECT_PF_SORT2);
-        CASE_CODE (PROF_DETECT_NONMPMLIST);
-        CASE_CODE (PROF_DETECT_TX);
-        CASE_CODE (PROF_DETECT_ALERT);
-        CASE_CODE (PROF_DETECT_TX_UPDATE);
-        CASE_CODE (PROF_DETECT_CLEANUP);
+        CASE_CODE(PROF_DETECT_SETUP);
+        CASE_CODE(PROF_DETECT_GETSGH);
+        CASE_CODE(PROF_DETECT_IPONLY);
+        CASE_CODE(PROF_DETECT_RULES);
+        CASE_CODE(PROF_DETECT_PF_PKT);
+        CASE_CODE(PROF_DETECT_PF_PAYLOAD);
+        CASE_CODE(PROF_DETECT_PF_TX);
+        CASE_CODE(PROF_DETECT_PF_RECORD);
+        CASE_CODE(PROF_DETECT_PF_SORT1);
+        CASE_CODE(PROF_DETECT_PF_SORT2);
+        CASE_CODE(PROF_DETECT_NONMPMLIST);
+        CASE_CODE(PROF_DETECT_TX);
+        CASE_CODE(PROF_DETECT_ALERT);
+        CASE_CODE(PROF_DETECT_TX_UPDATE);
+        CASE_CODE(PROF_DETECT_CLEANUP);
         default:
             return "UNKNOWN";
     }
@@ -1286,57 +1259,34 @@ const char * PacketProfileDetectIdToString(PacketProfileDetectId id)
  *
  * \retval string equivalent for the LoggerId id
  */
-const char * PacketProfileLoggertIdToString(LoggerId id)
+const char *PacketProfileLoggerIdToString(LoggerId id)
 {
     switch (id) {
-        CASE_CODE (LOGGER_UNDEFINED);
-        CASE_CODE (LOGGER_ALERT_DEBUG);
-        CASE_CODE (LOGGER_ALERT_FAST);
-        CASE_CODE (LOGGER_UNIFIED2);
-        CASE_CODE (LOGGER_ALERT_SYSLOG);
-        CASE_CODE (LOGGER_DROP);
-        CASE_CODE (LOGGER_JSON_ALERT);
-        CASE_CODE (LOGGER_JSON_ANOMALY);
-        CASE_CODE (LOGGER_JSON_DROP);
-        CASE_CODE (LOGGER_JSON_SSH);
-        CASE_CODE (LOGGER_DNS_TS);
-        CASE_CODE (LOGGER_DNS_TC);
-        CASE_CODE (LOGGER_JSON_SMB);
-        CASE_CODE (LOGGER_JSON_NFS);
-        CASE_CODE (LOGGER_HTTP);
-        CASE_CODE (LOGGER_JSON_DNS_TS);
-        CASE_CODE (LOGGER_JSON_DNS_TC);
-        CASE_CODE (LOGGER_JSON_DNP3_TS);
-        CASE_CODE (LOGGER_JSON_DNP3_TC);
-        CASE_CODE (LOGGER_JSON_HTTP);
-        CASE_CODE (LOGGER_JSON_DHCP);
-        CASE_CODE (LOGGER_JSON_KRB5);
-        CASE_CODE (LOGGER_JSON_IKEV2);
-        CASE_CODE (LOGGER_JSON_FTP);
-        CASE_CODE (LOGGER_JSON_TFTP);
-        CASE_CODE (LOGGER_JSON_SMTP);
-        CASE_CODE (LOGGER_JSON_SNMP);
-        CASE_CODE (LOGGER_JSON_TLS);
-        CASE_CODE (LOGGER_JSON_SIP);
-        CASE_CODE (LOGGER_JSON_TEMPLATE_RUST);
-        CASE_CODE (LOGGER_JSON_RFB);
-        CASE_CODE (LOGGER_JSON_MQTT);
-        CASE_CODE (LOGGER_JSON_TEMPLATE);
-        CASE_CODE (LOGGER_JSON_RDP);
-        CASE_CODE (LOGGER_JSON_DCERPC);
-        CASE_CODE (LOGGER_JSON_HTTP2);
-        CASE_CODE (LOGGER_TLS_STORE);
-        CASE_CODE (LOGGER_TLS);
-        CASE_CODE (LOGGER_FILE_STORE);
-        CASE_CODE (LOGGER_JSON_FILE);
-        CASE_CODE (LOGGER_TCP_DATA);
-        CASE_CODE (LOGGER_JSON_FLOW);
-        CASE_CODE (LOGGER_JSON_NETFLOW);
-        CASE_CODE (LOGGER_STATS);
-        CASE_CODE (LOGGER_JSON_STATS);
-        CASE_CODE (LOGGER_PRELUDE);
-        CASE_CODE (LOGGER_PCAP);
-        CASE_CODE (LOGGER_JSON_METADATA);
+        CASE_CODE(LOGGER_UNDEFINED);
+        CASE_CODE(LOGGER_HTTP);
+        CASE_CODE(LOGGER_TLS_STORE);
+        CASE_CODE(LOGGER_TLS);
+        CASE_CODE(LOGGER_JSON_TX);
+        CASE_CODE(LOGGER_FILE);
+        CASE_CODE(LOGGER_FILEDATA);
+        CASE_CODE(LOGGER_ALERT_DEBUG);
+        CASE_CODE(LOGGER_ALERT_FAST);
+        CASE_CODE(LOGGER_ALERT_SYSLOG);
+        CASE_CODE(LOGGER_JSON_ALERT);
+        CASE_CODE(LOGGER_JSON_ANOMALY);
+        CASE_CODE(LOGGER_JSON_DROP);
+        CASE_CODE(LOGGER_FILE_STORE);
+        CASE_CODE(LOGGER_JSON_FILE);
+        CASE_CODE(LOGGER_TCP_DATA);
+        CASE_CODE(LOGGER_JSON_FLOW);
+        CASE_CODE(LOGGER_JSON_NETFLOW);
+        CASE_CODE(LOGGER_STATS);
+        CASE_CODE(LOGGER_JSON_STATS);
+        CASE_CODE(LOGGER_PCAP);
+        CASE_CODE(LOGGER_JSON_METADATA);
+        CASE_CODE(LOGGER_JSON_FRAME);
+        CASE_CODE(LOGGER_JSON_STREAM);
+
         case LOGGER_SIZE:
             return "UNKNOWN";
     }
@@ -1455,4 +1405,73 @@ SCProfilingRegisterTests(void)
 #endif /* UNITTESTS */
 }
 
+void SCProfileRuleStartCollection(void)
+{
+}
+
+void SCProfileRuleStopCollection(void)
+{
+}
+
+#elif PROFILE_RULES
+
+thread_local int profiling_rules_entered = 0;
+int profiling_output_to_file = 0;
+static SC_ATOMIC_DECLARE(uint64_t, samples);
+static uint64_t rate = 0;
+static SC_ATOMIC_DECLARE(bool, profiling_rules_active);
+
+/**
+ * \brief Initialize profiling.
+ */
+void SCProfilingInit(void)
+{
+    SC_ATOMIC_INIT(profiling_rules_active);
+    SC_ATOMIC_INIT(samples);
+    intmax_t rate_v = 0;
+
+    (void)ConfGetInt("profiling.sample-rate", &rate_v);
+    if (rate_v > 0 && rate_v < INT_MAX) {
+        int literal_rate = (int)rate_v;
+        for (int i = literal_rate; i >= 1; i--) {
+            /* If i is a power of 2 */
+            if ((i & (i - 1)) == 0) {
+                rate = i - 1;
+                break;
+            }
+        }
+        if (rate != 0)
+            SCLogInfo("profiling runs for every %luth packet", rate + 1);
+        else
+            SCLogInfo("profiling runs for every packet");
+    }
+}
+
+/* see if we want to profile rules for this packet */
+int SCProfileRuleStart(Packet *p)
+{
+    if (!SC_ATOMIC_GET(profiling_rules_active)) {
+        return 0;
+    }
+    uint64_t sample = SC_ATOMIC_ADD(samples, 1);
+    if ((sample & rate) == 0) {
+        p->flags |= PKT_PROFILE;
+        return 1;
+    }
+
+    if (p->flags & PKT_PROFILE)
+        return 1;
+    return 0;
+}
+
+void SCProfileRuleStartCollection(void)
+{
+    SC_ATOMIC_SET(profiling_rules_active, true);
+}
+
+void SCProfileRuleStopCollection(void)
+{
+    SC_ATOMIC_SET(profiling_rules_active, false);
+}
+
 #endif /* PROFILING */
